//Stores the ID of the newly inserted row so we can add the DB generated ID to it in immediate update mode
var insertedRowRepeatId = null;

/**
 * Process any changes from the editable table.
 */
function processRecordChange(type, rowId)
{
    var repeatId = rowId.substring(0, rowId.length - 12);
    var contactId;
    if (dojo.byId(repeatId + 'CID'))
        contactId = dojo.byId(repeatId + 'CID').value;
        
    insertedRowRepeatId = null;
    
    //check whether to update the DB immediately or not
    if (hyf.util.getFieldValue('immediateUpdate') == 'true')
    {
        hyf.util.setFieldValue('recordID', contactId);
        if (type == 'insert')
        {
            hyf.util.setFieldValue('updateType', 'insert');
            //store the repeat ID for future use.
            insertedRowRepeatId = repeatId;
        }
        else if (type == 'remove')
        {
            hyf.util.setFieldValue('updateType', 'delete');
        }
        else if (type == 'edit')
        {
            hyf.util.setFieldValue('updateType', 'update');
        }
        else
        {
            //not a change we are interested in
            return;
        }
        
        //make the AJAX call to store the update in the database
        hyf.FMAction.handleAjaxSubmission( 
            {name: 'Action', option: 'Static', value: 'immediateUpdate.do'}, 
            {name: 'Source', option: 'AllFormData', value: ''}, 
            {name: 'Target', option: 'PageGroup', value: 'immediateUpdateResponse'}, 
            {name: 'Validate', option: 'Static', value: 'false'}, 
            {value: 'immediateUpdate'});
    }
    else
    {
        //not doing immediate updates, so only need to handle a delete operation.
        if (type == 'remove')
        {
            document.getElementById('deletedRows').value += '<record><CID>' + contactId + '</CID></record>';
        }
    }
}


/**
 * Ovverride the default mesasge displayed for the immediate update AJAX call so that it says
 * 'Saving...' rather than the default of 'Loading...'
 */
function immediateUpdateSetLoadingMessage(message)
{
    return '<span class="ajaxLoading">Saving...</span>';
}

/**
 * Handle the response fromt eh immediate update AJAX call.
 * If this was triggered following an insert (ie have an insertedRowRepeatId value)
 * then we need to ensure the new DB generated ID gets placed into the ID field for the new row.
 */
function immediateUpdateManipulateResponse(content, outcome)
{
    if (insertedRowRepeatId != null)
        return content + '<script type="text/javascript">hyf.util.setFieldValue("' + insertedRowRepeatId + 'CID", hyf.util.getFieldValue("newRecordId"));</script>';
    else
        return content;   
}